/*_PROGRAM__dvdt_zo___________________________________________________________*/

/*
         1         2         3         4         5         6         7         8
12345678901234567890123456789012345678901234567890123456789012345678901234567890


DESCRIPTION:

Calculate dv/dt and Zo from 50 Ohm load waveforms.


REVISION HISTORY:

08/22/00  Greg Edlund

*/

/*_IMPORT_____________________________________________________________________*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

/*_FUNCTIONS__________________________________________________________________*/

int read_file(char*,float**,float**);
float find_crossing(float*,float*,double,int,int,char*);
float *vector(int,int);
void free_vector(float*,int,int);

/*_MACROS_____________________________________________________________________*/

#define MAXBUF 120
#define TRUE 1
#define FALSE 0
#define RISE 1
#define FALL 0
#define SCALEX 1e-9

/*_DECLARATIONS_______________________________________________________________*/


main(int argc,char **argv)
{

   char buf[MAXBUF];         /*general purpose buffer*/
   char fname[MAXBUF];       /*file name*/
   int ndata;                /*number of data points in file*/
   float *x, *y;             /*waveform 1*/
   float xx20;               /*x value of 20% threshold crossing*/
   float xx80;               /*x value of 80% threshold crossing*/
   float vdd = 3.3;          /*Vdd supply voltage*/
   double vth20;             /*20% threshold*/
   double vth80;             /*80% threshold*/
   int edge = RISE;          /*rising or falling edge*/
   int i = 1;                /*data vector index*/

/*____________________________________________________________________________*/


   for (i=1; i<argc; ++i) strcpy(buf,argv[i]);

   /* if i >= argc print_help */

   for (i=1; i<argc; ++i) {
      if (strcmp(argv[i],"-f") == 0) {
         ++i;
         strcpy(fname,argv[i]);
      }
      if (strcmp(argv[i],"-vdd") == 0) {
         ++i;
         if (sscanf(argv[i],"%f",&vdd) != 1) {
            printf("-vdd argument must be floating point data.\n");
            exit(1);
         }
      }
      if (strcmp(argv[i],"-e") == 0) {
         ++i;
         if ((strcmp(argv[i],"r") == 0) || (strcmp(argv[i],"f") == 0)) {
            if (strcmp(argv[i],"r") == 0) {
               edge = RISE;
            }
            if (strcmp(argv[i],"f") == 0) {
               edge = FALL;
            }
         }
         else {
            printf("-e argument must be r or f.\n");
            exit(1);
         }
      }
   }

   ndata = read_file(fname,&x,&y);

   if (edge == RISE) {
      vth20 = 0.2*y[ndata];
      vth80 = 0.8*y[ndata];
   }
   else {
      vth20 = y[ndata] + 0.2*(y[1]-y[ndata]);
      vth80 = y[ndata] + 0.8*(y[1]-y[ndata]);
   }

   xx20 = find_crossing(x,y,vth20,ndata,edge,fname);
   xx80 = find_crossing(x,y,vth80,ndata,edge,fname);
   printf("\n");
   printf("20 pct point = %11.4e ns, %11.4e V\n",xx20,vth20);
   printf("80 pct point = %11.4e ns, %11.4e V\n\n",xx80,vth80);

   printf("dv    = %11.4e V\n",(fabs(vth80-vth20)));
   printf("dt    = %11.4e ns\n",(fabs(xx20-xx80)));
   printf("dv/dt = %11.4e V/ns\n\n",(fabs((vth80-vth20)/(xx20-xx80))));

   if (edge == RISE) {
      printf("Zout  = %11.4e Ohms\n",(50*(vdd-y[ndata])/y[ndata]));
   }
   else {
      printf("Zout  = %11.4e Ohms\n",(50*y[ndata]/(vdd-y[ndata])));
   }
   printf("\n");
/*
** free space.
*/
   free_vector(x,1,ndata);
   free_vector(y,1,ndata);

   return(0);

}

/*_FUNCTIONS__________________________________________________________________*/


int read_file(fname,px,py)
char *fname;
float **px;
float **py;
{

   char buf[MAXBUF];         /*character buffer*/
   FILE *fptr;               /*file stream pointer*/
   int line = 0;             /*current line number*/
   float dum1,dum2;          /*dummy for reading real data with sscanf*/
   int dataerror = FALSE;    /*data error flag*/
   float *x, *y;             /*data vectors*/

/*
** open file and count lines.
*/
   fptr = fopen(fname,"r");
   if (fptr == NULL) {
      printf("Cannot open file %s.\n",fname);
      exit(1);
   }
   while (fgets(buf,MAXBUF,fptr) != NULL) {
      ++line;
   }
   if (line == 0) {
      printf("Empty data file %s.\n",fname);
      exit(1);
   }

/*
** allocate space for data storage.
*/
   x = vector(1,line);
   y = vector(1,line);

/*
** return pointer to beginning of file.
*/
   fseek(fptr,0L,0);
   line = 0;

/*
** read two columns of floating point data and close file.
** scale x values and reset x vector to x[1] = 0.
*/
   while (fgets(buf,MAXBUF,fptr) != NULL) {
      ++line;
      if (sscanf(buf,"%f %f",&dum1,&dum2) != 2) {
         printf("Error in floating point data on line %d.\n",line);
         dataerror = TRUE;
      }
      else {
         if (line == 1) {
            x[line] = 0;
            y[line] = dum2;
         }
         else {
            x[line] = (dum1/SCALEX) - x[1];
            y[line] = dum2;
         }
      }
      if (dataerror == TRUE) exit(1);
   }

/*
** close file.
** return pointers to new vectors and number of lines.
*/
   fclose(fptr);
   *px = x;
   *py = y;
   return(line);

}

/*____________________________________________________________________________*/


float find_crossing(x,y,yth,ndata,edge,fname)
float *x;
float *y;
double yth;
int ndata;
int edge;
char *fname;
{

   int i;                    /*data vector index*/
   float m;                  /*slope*/
   float b;                  /*intercept*/
   float xx;                 /*x value of threshold crossing*/
   int found_xx = FALSE;     /*threshold flag*/

/*
** loop through data vectors and look for threshold crossing.
** if crossing is found, use linear interpolation to find y value.
** y = mx + b, where y is unknown.
*/
   for (i=1; i<ndata; ++i) {
      if (y[i] == yth) {
         xx = x[i];
         found_xx = TRUE;
      }
      else {
         if (edge == RISE) {
            if ((y[i]<=yth)&&(y[i+1]>yth)) {
               m = (y[i+1]-y[i])/(x[i+1]-x[i]);
               b = y[i]-(m*x[i]);
               xx = (yth-b)/m;
               found_xx = TRUE;
            }
         }
         if (edge == FALL) {
            if ((y[i]>=yth)&&(y[i+1]<yth)) {
               m = (y[i+1]-y[i])/(x[i+1]-x[i]);
               b = y[i]-(m*x[i]);
               xx = (yth-b)/m;
               found_xx = TRUE;
            }
         }
      }
   }
   if (found_xx == FALSE) {
      printf("No threshold crossing found in %s.\n",fname);
      exit(1);
   }
   return(xx);
}

/*____________________________________________________________________________*/

float *vector(int nl,int nh)
{
   float *v;
   v = (float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
   if (!v) {
      printf("allocation failure in vector().\n");
      exit(1);
   }
   return v-nl;
}

/*____________________________________________________________________________*/


void free_vector(float *v,int nl,int nh)
{
   free((char*) (v+nl));
}
